--[[-------------------------------------------------------------------
X-Tensions Universal PA Keyboard (XP-950) Driver

Most probably compatible with SnapNType T806 (XP-950 uses Snap'NType drivers on
Palm OS).
http://www.x-tensions.net/

9600 baud, 8N1

-----------

Copyright (c) 2007, Peter Novak <support [at] aronde.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-------------------------------------------------------------------]]--

-- Basic information about this driver
description = "SnapNType T806 IR Keyboard Driver"
version = "0.1"

-- Modifier key information
scancode_fn = 0x1E
scancode_punc = 0
scancode_num = 0
scancode_shiftl = 0x10
scancode_shiftr = 0x12
scancode_caps = 0x14
has_num_lock = 0
has_punc_lock = 0
has_fn_lock = 0

-- Scancode maps
map_normal = {
	   0; 0; 0; 0; 0; 0; 0;
	ZK_BACKSPACE; ZK_TAB; 0; 0; 0; ZK_RETURN; 0; 0;
	ZK_SHIFT; ZK_CONTROLL; ZK_SHIFT; ZK_DELETE; 0; 0; 0; 0;
	0; 0; 0; ZK_TILDE; ZK_ESCAPE; ZK_ALTL; 0; 0;
	ZK_SPACE; ZK_SPACE; 0; 0; 0; ZK_RIGHT; ZK_UP; ZK_LEFT;
	ZK_DOWN; 0; 0; 0; ZK_APOSTROPHE; ZK_MINUS; ZK_PERIOD; ZK_SLASH;
	ZK_0; ZK_1; ZK_2; ZK_3; ZK_4; ZK_5; ZK_6; ZK_7;
	ZK_8; ZK_9; 0; ZK_SEMICOLON; ZK_COMMA; ZK_EQUAL; 0; 0;
	0; 0; ZK_B; ZK_C; ZK_D; ZK_E; ZK_F; ZK_G;
	ZK_H; ZK_I; ZK_J; ZK_K; ZK_L; ZK_M; ZK_N; ZK_O;
	ZK_P; ZK_Q; ZK_R; ZK_S; ZK_T; ZK_U; ZK_V; ZK_W;
	ZK_X; ZK_Y; ZK_Z; ZK_BRACKETLEFT; ZK_BACKSLASH; ZK_BRACKETRIGHT; ZK_A; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; ZK_ACTIVITY; ZK_CONTACTS; ZK_F6; ZK_F7; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0 }


map_shift = {
	   0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0;
	0; 0; 0; 0; 0; 0; 0; 0 }


map_punc = {0}

map_num = {0}

map_fn = {0}

--[[-------------------------------------------------------------------
The SnapNType generates 4 bytes for a key press and four bytes for key release.
The sequence starts with 0xC0 (192) and ends with 0xC1 (193). In between is a
group code and key code - in the key down sequence. In the key up sequence, the
group and key codes are the same (I do not care how it is different from key
down - unimportant.

Otherwise the mechanics is similar to Belkin keyboard, however not the same
state machine can be used as group codes can be also 0xC0/0xC1 (QWERT).
-------------------------------------------------------------------]]--
SNT_IDLE	= 0
SNT_GROUP	= 1
SNT_KEY		= 2
SNT_END		= 3

snt_state 	= SNT_IDLE 

SNT_KEYDOWN	= 1
SNT_KEYUP	= 0

group_code	= 0
key_code	= 0
	
function do_scancode( s )
	-- print("SNT_STATE = ", snt_state)

	key = 0
	key_down = 0

	if (snt_state == SNT_IDLE) then
		if (s == 0xC0) then
			-- sequence starts
			snt_state = SNT_GROUP
			
			return 0, 0
		else
			-- discard the code
			return 0, 0
		end
	elseif (snt_state == SNT_GROUP) then
		-- read the group code
		group_code = s
		snt_state = SNT_KEY

		return 0, 0
	elseif (snt_state == SNT_KEY) then
		-- read the key code
		key_code = s & 0x7f
		snt_state = SNT_END

		-- decide whether this is key up event, or key down event
		--if (group_code == s) then
		--	return key_code, SNT_KEYUP
		--else
		--	return key_code, SNT_KEYDOWN
		--end
		if (s & 0x80) == 0 then
			return key_code, SNT_KEYDOWN
		else
			return key_code, SNT_KEYUP
		end
	else
		-- if (snt_state == SNT_END) then
		-- we reached the fourth byte - should be 0xC1, but we do not check
		key_code = 0
		group_code = 0
		snt_state = SNT_IDLE

		return 0, 0
	end
end

